@@ -1,12 +1,6 @@
 //! Various modifiers for components.
 
-#[cfg(feature = "alloc")]
-use alloc::string::String;
-#[cfg(feature = "alloc")]
-use core::mem;
-
-#[cfg(feature = "alloc")]
-use crate::{error::InvalidFormatDescription, format_description::helper};
+use core::num::NonZeroU16;
 
 // region: date modifiers
 /// Day of the month.
@@ -107,10 +101,27 @@ pub struct WeekNumber {
 pub enum YearRepr {
     /// The full value of the year.
     Full,
+    /// All digits except the last two. Includes the sign, if any.
+    Century,
     /// Only the last two digits of the year.
     LastTwo,
 }
 
+/// The range of years that are supported.
+///
+/// This modifier has no effect when the year repr is [`LastTwo`](YearRepr::LastTwo).
+#[non_exhaustive]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum YearRange {
+    /// Years between -9999 and 9999 are supported.
+    Standard,
+    /// Years between -999_999 and 999_999 are supported, with the sign being required if the year
+    /// contains more than four digits.
+    ///
+    /// If the `large-dates` feature is not enabled, this variant is equivalent to `Standard`.
+    Extended,
+}
+
 /// Year of the date.
 #[non_exhaustive]
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -119,6 +130,8 @@ pub struct Year {
     pub padding: Padding,
     /// What kind of representation should be used?
     pub repr: YearRepr,
+    /// What range of years is supported?
+    pub range: YearRange,
     /// Whether the value is based on the ISO week number or the Gregorian calendar.
     pub iso_week_based: bool,
     /// Whether the `+` sign is present when a positive year contains fewer than five digits.
@@ -242,6 +255,56 @@ pub enum Padding {
     None,
 }
 
+/// Ignore some number of bytes.
+///
+/// This has no effect when formatting.
+#[non_exhaustive]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct Ignore {
+    /// The number of bytes to ignore.
+    pub count: NonZeroU16,
+}
+
+// Needed as `Default` is deliberately not implemented for `Ignore`. The number of bytes to ignore
+// must be explicitly provided.
+impl Ignore {
+    /// Create an instance of `Ignore` with the provided number of bytes to ignore.
+    pub const fn count(count: NonZeroU16) -> Self {
+        Self { count }
+    }
+}
+
+/// The precision of a Unix timestamp.
+#[non_exhaustive]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum UnixTimestampPrecision {
+    /// Seconds since the Unix epoch.
+    Second,
+    /// Milliseconds since the Unix epoch.
+    Millisecond,
+    /// Microseconds since the Unix epoch.
+    Microsecond,
+    /// Nanoseconds since the Unix epoch.
+    Nanosecond,
+}
+
+/// A Unix timestamp.
+#[non_exhaustive]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct UnixTimestamp {
+    /// The precision of the timestamp.
+    pub precision: UnixTimestampPrecision,
+    /// Whether the `+` sign must be present for a non-negative timestamp.
+    pub sign_is_mandatory: bool,
+}
+
+/// The end of input.
+///
+/// There is currently not customization for this modifier.
+#[non_exhaustive]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct End;
+
 /// Generate the provided code if and only if `pub` is present.
 macro_rules! if_pub {
     (pub $(#[$attr:meta])*; $($x:tt)*) => {
@@ -317,12 +380,15 @@ impl_const_default! {
     };
     /// Creates a modifier that indicates the value uses the [`Full`](Self::Full) representation.
     YearRepr => Self::Full;
+    /// Creates a modifier that indicates the value uses the [`Extended`](Self::Extended) range.
+    YearRange => Self::Extended;
     /// Creates a modifier that indicates the value uses the [`Full`](YearRepr::Full)
     /// representation, is [padded with zeroes](Padding::Zero), uses the Gregorian calendar as its
     /// base, and only includes the year's sign if necessary.
     @pub Year => Self {
         padding: Padding::Zero,
         repr: YearRepr::Full,
+        range: YearRange::Extended,
         iso_week_based: false,
         sign_is_mandatory: false,
     };
@@ -348,10 +414,10 @@ impl_const_default! {
     /// Creates a modifier that indicates the stringified value contains [one or more
     /// digits](SubsecondDigits::OneOrMore).
     @pub Subsecond => Self { digits: SubsecondDigits::OneOrMore };
-    /// Creates a modifier that indicates the value uses the `+` sign for all positive values
-    /// and is [padded with zeroes](Padding::Zero).
+    /// Creates a modifier that indicates the value only uses a sign for negative values and is
+    /// [padded with zeroes](Padding::Zero).
     @pub OffsetHour => Self {
-        sign_is_mandatory: true,
+        sign_is_mandatory: false,
         padding: Padding::Zero,
     };
     /// Creates a modifier that indicates the value is [padded with zeroes](Padding::Zero).
@@ -360,168 +426,15 @@ impl_const_default! {
     @pub OffsetSecond => Self { padding: Padding::Zero };
     /// Creates a modifier that indicates the value is [padded with zeroes](Self::Zero).
     Padding => Self::Zero;
-}
-
-/// The modifiers parsed for any given component. `None` indicates the modifier was not present.
-#[allow(clippy::missing_docs_in_private_items)] // fields
-#[derive(Debug, Default)]
-pub(crate) struct Modifiers {
-    pub(crate) padding: Option<Padding>,
-    pub(crate) hour_is_12_hour_clock: Option<bool>,
-    pub(crate) period_is_uppercase: Option<bool>,
-    pub(crate) month_repr: Option<MonthRepr>,
-    pub(crate) subsecond_digits: Option<SubsecondDigits>,
-    pub(crate) weekday_repr: Option<WeekdayRepr>,
-    pub(crate) weekday_is_one_indexed: Option<bool>,
-    pub(crate) week_number_repr: Option<WeekNumberRepr>,
-    pub(crate) year_repr: Option<YearRepr>,
-    pub(crate) year_is_iso_week_based: Option<bool>,
-    pub(crate) sign_is_mandatory: Option<bool>,
-    pub(crate) case_sensitive: Option<bool>,
-}
-
-impl Modifiers {
-    /// Parse the modifiers of a given component.
-    #[cfg(feature = "alloc")]
-    #[allow(clippy::too_many_lines)]
-    pub(crate) fn parse(
-        component_name: &[u8],
-        mut bytes: &[u8],
-        index: &mut usize,
-    ) -> Result<Self, InvalidFormatDescription> {
-        let mut modifiers = Self::default();
-
-        while !bytes.is_empty() {
-            // Trim any whitespace between modifiers.
-            bytes = helper::consume_whitespace(bytes, index);
-
-            let modifier;
-            if let Some(whitespace_loc) = bytes.iter().position(u8::is_ascii_whitespace) {
-                *index += whitespace_loc;
-                modifier = &bytes[..whitespace_loc];
-                bytes = &bytes[whitespace_loc..];
-            } else {
-                modifier = mem::take(&mut bytes);
-            }
-
-            if modifier.is_empty() {
-                break;
-            }
-
-            #[allow(clippy::unnested_or_patterns)]
-            match (component_name, modifier) {
-                (b"day", b"padding:space")
-                | (b"hour", b"padding:space")
-                | (b"minute", b"padding:space")
-                | (b"month", b"padding:space")
-                | (b"offset_hour", b"padding:space")
-                | (b"offset_minute", b"padding:space")
-                | (b"offset_second", b"padding:space")
-                | (b"ordinal", b"padding:space")
-                | (b"second", b"padding:space")
-                | (b"week_number", b"padding:space")
-                | (b"year", b"padding:space") => modifiers.padding = Some(Padding::Space),
-                (b"day", b"padding:zero")
-                | (b"hour", b"padding:zero")
-                | (b"minute", b"padding:zero")
-                | (b"month", b"padding:zero")
-                | (b"offset_hour", b"padding:zero")
-                | (b"offset_minute", b"padding:zero")
-                | (b"offset_second", b"padding:zero")
-                | (b"ordinal", b"padding:zero")
-                | (b"second", b"padding:zero")
-                | (b"week_number", b"padding:zero")
-                | (b"year", b"padding:zero") => modifiers.padding = Some(Padding::Zero),
-                (b"day", b"padding:none")
-                | (b"hour", b"padding:none")
-                | (b"minute", b"padding:none")
-                | (b"month", b"padding:none")
-                | (b"offset_hour", b"padding:none")
-                | (b"offset_minute", b"padding:none")
-                | (b"offset_second", b"padding:none")
-                | (b"ordinal", b"padding:none")
-                | (b"second", b"padding:none")
-                | (b"week_number", b"padding:none")
-                | (b"year", b"padding:none") => modifiers.padding = Some(Padding::None),
-                (b"hour", b"repr:24") => modifiers.hour_is_12_hour_clock = Some(false),
-                (b"hour", b"repr:12") => modifiers.hour_is_12_hour_clock = Some(true),
-                (b"month", b"case_sensitive:true")
-                | (b"period", b"case_sensitive:true")
-                | (b"weekday", b"case_sensitive:true") => modifiers.case_sensitive = Some(true),
-                (b"month", b"case_sensitive:false")
-                | (b"period", b"case_sensitive:false")
-                | (b"weekday", b"case_sensitive:false") => modifiers.case_sensitive = Some(false),
-                (b"month", b"repr:numerical") => modifiers.month_repr = Some(MonthRepr::Numerical),
-                (b"month", b"repr:long") => modifiers.month_repr = Some(MonthRepr::Long),
-                (b"month", b"repr:short") => modifiers.month_repr = Some(MonthRepr::Short),
-                (b"offset_hour", b"sign:automatic") | (b"year", b"sign:automatic") => {
-                    modifiers.sign_is_mandatory = Some(false);
-                }
-                (b"offset_hour", b"sign:mandatory") | (b"year", b"sign:mandatory") => {
-                    modifiers.sign_is_mandatory = Some(true);
-                }
-                (b"period", b"case:upper") => modifiers.period_is_uppercase = Some(true),
-                (b"period", b"case:lower") => modifiers.period_is_uppercase = Some(false),
-                (b"subsecond", b"digits:1") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::One);
-                }
-                (b"subsecond", b"digits:2") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Two);
-                }
-                (b"subsecond", b"digits:3") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Three);
-                }
-                (b"subsecond", b"digits:4") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Four);
-                }
-                (b"subsecond", b"digits:5") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Five);
-                }
-                (b"subsecond", b"digits:6") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Six);
-                }
-                (b"subsecond", b"digits:7") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Seven);
-                }
-                (b"subsecond", b"digits:8") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Eight);
-                }
-                (b"subsecond", b"digits:9") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::Nine);
-                }
-                (b"subsecond", b"digits:1+") => {
-                    modifiers.subsecond_digits = Some(SubsecondDigits::OneOrMore);
-                }
-                (b"weekday", b"repr:short") => modifiers.weekday_repr = Some(WeekdayRepr::Short),
-                (b"weekday", b"repr:long") => modifiers.weekday_repr = Some(WeekdayRepr::Long),
-                (b"weekday", b"repr:sunday") => modifiers.weekday_repr = Some(WeekdayRepr::Sunday),
-                (b"weekday", b"repr:monday") => modifiers.weekday_repr = Some(WeekdayRepr::Monday),
-                (b"weekday", b"one_indexed:true") => modifiers.weekday_is_one_indexed = Some(true),
-                (b"weekday", b"one_indexed:false") => {
-                    modifiers.weekday_is_one_indexed = Some(false);
-                }
-                (b"week_number", b"repr:iso") => {
-                    modifiers.week_number_repr = Some(WeekNumberRepr::Iso);
-                }
-                (b"week_number", b"repr:sunday") => {
-                    modifiers.week_number_repr = Some(WeekNumberRepr::Sunday);
-                }
-                (b"week_number", b"repr:monday") => {
-                    modifiers.week_number_repr = Some(WeekNumberRepr::Monday);
-                }
-                (b"year", b"repr:full") => modifiers.year_repr = Some(YearRepr::Full),
-                (b"year", b"repr:last_two") => modifiers.year_repr = Some(YearRepr::LastTwo),
-                (b"year", b"base:calendar") => modifiers.year_is_iso_week_based = Some(false),
-                (b"year", b"base:iso_week") => modifiers.year_is_iso_week_based = Some(true),
-                _ => {
-                    return Err(InvalidFormatDescription::InvalidModifier {
-                        value: String::from_utf8_lossy(modifier).into_owned(),
-                        index: *index,
-                    });
-                }
-            }
-        }
-
-        Ok(modifiers)
-    }
+    /// Creates a modifier that indicates the value represents the [number of seconds](Self::Second)
+    /// since the Unix epoch.
+    UnixTimestampPrecision => Self::Second;
+    /// Creates a modifier that indicates the value represents the [number of
+    /// seconds](UnixTimestampPrecision::Second) since the Unix epoch. The sign is not mandatory.
+    @pub UnixTimestamp => Self {
+        precision: UnixTimestampPrecision::Second,
+        sign_is_mandatory: false,
+    };
+    /// Creates a modifier used to represent the end of input.
+    @pub End => End;
 }
